-- -----------------------------------------------------
-- OpenEstate-ImmoTool
-- update 2 for HSQLDB
-- Copyright (C) 2009-2019 OpenEstate.org
-- -----------------------------------------------------
-- Add sequence 'seq_immotool_logs'
-- Add table 'immotool_logs'
-- Add table 'immotool_logs_changes'
-- Add view 'view_immotool_logs_changes'
-- Add procedure 'remove_immotool_logs'
-- Add procedure 'remove_immotool_logs_changes'
-- Add procedure 'save_immotool_logs'
-- Add procedure 'save_immotool_logs_changes'
-- -----------------------------------------------------

CREATE SEQUENCE seq_immotool_logs
  AS BIGINT START WITH 1 INCREMENT BY 1 NO CYCLE;


-- -----------------------------------------------------
-- Table immotool_logs
-- -----------------------------------------------------
CREATE CACHED TABLE immotool_logs (
  log_id BIGINT GENERATED BY DEFAULT AS SEQUENCE seq_immotool_logs NOT NULL,
  log_entry_type VARCHAR(255) NOT NULL,
  log_entry_action VARCHAR(255) DEFAULT NULL NULL,
  log_text LONGVARCHAR DEFAULT NULL NULL,
  log_notes LONGVARCHAR DEFAULT NULL NULL,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  created_by VARCHAR(255) DEFAULT NULL NULL,
  modified_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  modified_by VARCHAR(255) DEFAULT NULL NULL,
  PRIMARY KEY(log_id)
);


-- -----------------------------------------------------
-- Table immotool_logs_changes
-- -----------------------------------------------------
CREATE CACHED TABLE immotool_logs_changes (
  log_id BIGINT NOT NULL,
  change_key VARCHAR(255) NOT NULL,
  change_value_old LONGVARCHAR DEFAULT NULL NULL,
  change_value_new LONGVARCHAR DEFAULT NULL NULL,
  PRIMARY KEY (log_id, change_key),
  FOREIGN KEY (log_id)
    REFERENCES immotool_logs(log_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-- -----------------------------------------------------
-- View view_immotool_logs_changes
-- -----------------------------------------------------
CREATE VIEW view_immotool_logs_changes AS
  SELECT * FROM immotool_logs_changes;

GRANT SELECT
  ON view_immotool_logs_changes
  TO "IMMOTOOL";


\.
-- -----------------------------------------------------
-- Procedure remove_immotool_logs
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_logs( val_log_id BIGINT )
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Protokolleintrag entfernen
    DELETE
      FROM immotool_logs
      WHERE log_id = val_log_id;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure remove_immotool_logs_changes
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_logs_changes( val_log_id BIGINT, val_change_key VARCHAR(255) )
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- Alle Änderungen de Logs entfernen
	  IF val_change_key IS NULL THEN

	    DELETE
	      FROM immotool_logs_changes
	      WHERE log_id = val_log_id;

		-- Spezifische Änderung des Logs entfernen
	  ELSE

	    DELETE
	      FROM immotool_logs_changes
	      WHERE log_id = val_log_id
	      AND change_key = val_change_key;

  	END IF;

  END;
.;

GRANT EXECUTE
  ON PROCEDURE remove_immotool_logs_changes
  TO "IMMOTOOL";


\.
-- -----------------------------------------------------
-- Procedure save_immotool_logs
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_logs(
  INOUT val_log_id BIGINT,
  val_log_entry_type VARCHAR(255),
  val_log_entry_action VARCHAR(255),
  val_log_text LONGVARCHAR,
  val_log_notes LONGVARCHAR,
  val_created_by VARCHAR(255),
  val_modified_by VARCHAR(255) )
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- Neuer Protokolleintrag
    IF val_log_id < 1 THEN

      -- Protokolleintrag speichern
      INSERT INTO immotool_logs (
        log_entry_type,
        log_entry_action,
        log_text,
        log_notes,
        created_by,
        modified_by )
      VALUES (
        val_log_entry_type,
        val_log_entry_action,
        val_log_text,
        val_log_notes,
        val_created_by,
        val_modified_by );

      -- ID zurückliefern
      SET val_log_id = IDENTITY();

    -- Bestehender Protokolleintrag
    ELSE

      -- Protokolleintrag bearbeiten
      UPDATE immotool_logs
        SET
          log_entry_type = val_log_entry_type,
          log_entry_action = val_log_entry_action,
          log_text = val_log_text,
          log_notes = val_log_notes,
          created_by = val_created_by,
          modified_by = val_modified_by,
          modified_at = NOW()
        WHERE
          log_id = val_log_id;

    END IF;

  END;
.;

GRANT EXECUTE
  ON PROCEDURE save_immotool_logs
  TO "IMMOTOOL";


\.
-- -----------------------------------------------------
-- Procedure save_immotool_logs_changes
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_logs_changes(
  val_log_id BIGINT,
  val_change_key VARCHAR(255),
  val_change_value_old LONGVARCHAR,
  val_change_value_new LONGVARCHAR )
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- Existierende Änderung entfernen, wenn vorhanden
    DELETE
      FROM immotool_logs_changes
      WHERE log_id = val_log_id
      AND change_key = val_change_key;

    -- Änderung speichern
    INSERT INTO immotool_logs_changes (
      log_id,
      change_key,
      change_value_old,
      change_value_new )
    VALUES (
      val_log_id,
      val_change_key,
      val_change_value_old,
      val_change_value_new );

  END;
.;

GRANT EXECUTE
  ON PROCEDURE save_immotool_logs_changes
  TO "IMMOTOOL";
